{
  "Peers": [{% if relay_peer_uri %}"tcp://{{ relay_peer_uri }}:{{ relay_peer_port }}"{% endif %}],
  "InterfacePeers": {},
  "Listen": [
    "{{ ygg_listen_uri }}"
  ],
  "MulticastInterfaces": [],
  "AllowedPublicKeys": [],
  "IfName": "ygg0",
  "IfMTU": 65535,
  "NodeInfoPrivacy": true,
  "PrivateKeyPath": "{{ ygg_key_file }}",
  "NodeInfo": {
    "role": "jrun-host-bootstrap",
    "bridge_ip": "{{ jrun_bridge_ip }}"
  }
}
