set daemon 30
set logfile syslog
set idfile /var/db/monit.id
set pidfile /var/run/monit.pid
set statefile /var/db/monit.state
set httpd port 2812 and
    use address {{ item.ip }}
    allow {{ item.ip }}

check system {{ item.name }}-system
    if loadavg(5min) > 6 for 5 cycles then alert
    if memory usage > 90% for 5 cycles then alert
    if cpu usage (system) > 95% for 5 cycles then alert

include /usr/local/etc/monit.d/*.monitrc
