{
  "PrivateKeyPath": "{{ ygg_key_file }}",
  "Peers": [
    "{{ ygg_host_peer_uri }}"
  ],
  "Listen": [],
  "AdminListen": "unix:///var/run/yggdrasil.sock",
  "MulticastInterfaces": [],
  "IfName": "ygg0"
}
